/******************************************************************
*  rot.c                                                          *
*                                                                 *
*  a simple implementation of the rot13 [en,de]coding scheme.     *
*                                                                 *
* copyright (C) 1997 Shawn Boyette                                *
* this software is free and is distributed under the terms of the *
* GNU General Public License.  See the file COPYING for more info *
*                                                                 *
* usage: rot13 [options] <filename>                               *
*        runs <filename> through a rot13 code routine and writes  *
*        out the crypted file                                     *
*                                                                 *
* version 0.9 - took input from prompt, coded it and printf'ed it *
* version 1.0 - runs over input file, accepts -o, -q, -h args     *
* version 1.1 - added -d arg, fixed problem with [,\,],^,_,and `  *
* version 1.2 - added -c, -u , -r args                            *
******************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

char    *program_name;       /* name of the program */
char    delete_str[100];     /* command string for system() call */
int     user_output=0;       /* switch for -o option */
int     quiet_mode=0;        /* switch for -q option */
int     delete_flag=0;       /* switch for -d option */
int     checksum_flag=0;     /* switch for -c option */
int     unchksum_flag=0;     /* switch for -u option */
int     runtime_flag = 0;    /* switch for -r option */
char    checksum;            /* value of checksum */
int     char_in = 0;         /* character read from input file */
float   total_chars = 0;     /* counter for chars in file */
float   chars_conv = 0;      /* counter for converted chars */
float   percent = 0;         /* percentage converted */
FILE    *in_file;            /* input file */
FILE    *out_file;           /* output file */

main(int argc, char *argv[]) {

  /* declare functions */
  char ndcode(char translator);  /* conversion routine */
  char calcsum(void);            /* calculates checksum */
  void help(void);               /* onscreen help */

  /* tell the program it's name */
  program_name = argv[0];

  /* see if a file was specified. . . */
  if(argc == 1)
    help();

  /*
   * parse command line args
   */
  while((argc > 1) && (argv[1][0] == '-')) {
    switch(argv[1][1]) {
      case 'q':
        quiet_mode = 1;
        break;
      case 'o':
        out_file = fopen(&argv[1][2], "w");
        user_output = 1;
        break;
      case 'd':
        delete_flag = 1;
        break;
      case 'r':
        runtime_flag = 1;
        break;
      case 'u':
        unchksum_flag = 1;
        checksum = atoi(&argv[1][2]);
        break;
      case 'c':
        checksum_flag = 1;
        break;
      case 'h':
        help();
      default:
        (void)fprintf(stderr,"Bad option %s\n", argv[1]);
        help();
    }
    argv++;    /* move arg list up by 1 */
    argc--;    /* decrement arg count */
  }

  /* see if <infile> exists */
  in_file = fopen(argv[1], "r");
  if(in_file == NULL) {
    (void)printf("Hmmm...%s does not seem to exist.\n", argv[1]);
    exit(8);
  }

  /* calculate checksum */
  if(checksum_flag) {
    checksum = calcsum();
    in_file = fopen(argv[1], "r");
  }

  /* open output file if neccessary */
  if(!user_output)
    out_file = fopen("output.rot", "w");

  /* crypt file */
  while(1) {
    char_in = fgetc(in_file);
    if(char_in == EOF)
      break;
    total_chars++;
    fputc(ndcode(char_in), out_file);
  }

  /* delete input file if -d switch thrown */
  if(delete_flag) {
#ifdef unix
    (void)strcpy(delete_str, "rm ");
#endif
#ifdef __MSDOS__
    (void)strcpy(delete_str, "del ");
#endif
    (void)strcat(delete_str, argv[1]);
    system(delete_str);
  }

  /* print stats if not quiet mode */
  if(!quiet_mode) {
    percent = (chars_conv / total_chars) * 100.0;

    (void)printf("\nTotal characters read: %f\n", total_chars);
    (void)printf("Characters encoded   : %f\n", chars_conv);
    if(runtime_flag)
      (void)printf("Elapsed Clock Ticks  : %u\n", clock());
    (void)printf("Encryption percentage: %f\n\n", percent);
    if(checksum_flag)
      (void)printf("Decrypt Key          : %d\n\n", checksum);
  }

  (void)fclose(in_file);
  (void)fclose(out_file);
  return (0);
}



/*************************************
* ndcode - rot13 codec function      *
*                                    *
* parameters -                       *
*   translator -- char to be encoded *
*                 or decoded         *
*                                    *
* returns -                          *
*   the input character, run through *
*     rot13 coding                   *
*************************************/

char ndcode(char trans) {

  /* take care of -u option */
  if(unchksum_flag) {
    /*printf("%d ", (trans - checksum));*/
    if(((trans - checksum) <= -134 && (trans - checksum) >= -159) ||
       ((trans - checksum) <= -166 && (trans - checksum) >= -191))
      trans -= checksum;
  }

  /* only encode letters... */
  if ((trans >= 65 && trans <= 90) || (trans >= 97 && trans <= 122)) {
    /*
     * rollback 26 if 'N' to 'Z' OR 'n' to 'z'
     * this will rot letters inside their own case
     * as per the example in the Hacker Jargon File 4.0
     */
    if (trans >= 78 && trans <= 90)
      trans -= 26;
    if (trans >= 110 && trans <= 122)
      trans -= 26;

    /* do rot13 encoding */
    trans += 13;
    chars_conv++;
    /*printf("%c %d - ", trans, trans);*/

    /* handle -c encoding */
    if(checksum_flag) {
      trans += checksum;
      /*printf("%c - %d\n", trans, trans);*/
    }
  }
  return (trans);
}



/*****************************
* calcsum - calculate        *
*           checksum digit   *
*           from val of 1st  *
*           5 chars of file  *
*                            *
* parameters - none          *
*                            *
* returns - chksum           *
*****************************/

char calcsum(void) {

  int counter;
  int chksum = 0;
  int current_char;

  for(counter=0;counter < 5; counter++) {
    current_char = fgetc(in_file);
    chksum += current_char;
  }

  chksum /= 3;

  while(chksum < 65)
    chksum += 17;

  while(chksum > 127)
    chksum -= 23;

  (void)fclose(in_file);
  return(chksum);
}



/*****************************
* help - onscreen help func  *
*                            *
* prints out usage and       *
* options then exits with    *
* error return code          *
*****************************/

void help(void) {
  (void)fprintf(stderr,"Usage is %s [options] <infile>\n",
                program_name);
  (void)fprintf(stderr,"Options:\n");
  (void)fprintf(stderr,"    -h            Print this message\n");
  (void)fprintf(stderr,"    -q            Quiet mode (Don't print statistics)\n");
  (void)fprintf(stderr,"    -o<filename>  Specify output filename\n");
  (void)fprintf(stderr,"                  Default is output.rot\n");
  (void)fprintf(stderr,"    -d            Delete source file\n");
  (void)fprintf(stderr,"    -c            Use modified rot13 cipher\n");
  (void)fprintf(stderr,"    -u<key>       Decrypt files made with -c\n");
  (void)fprintf(stderr,"    -r            Report runtime in clock ticks\n");

  exit(5);
}
